import os
import sys
import shutil
import subprocess
import http.cookiejar
import urllib.parse
from configparser import SafeConfigParser
from distutils.util import strtobool

USER_AGENT='Mozilla/5.0 (Windows NT 6.1; rv:30.0) Gecko/20100101 Firefox/30.0'

def yes_no_query(question):
    sys.stdout.write('{} (y/n) '.format(question)),
    while True:
        try:
            return strtobool(input().lower())
        except ValueError:
            print("Please respond with 'y' or 'n'.")

def move_ask_overwrite(src, dest):
    if os.path.exists(dest):
        if yes_no_query("File '{}' already exists. Overwrite file?".format(dest)):
            os.remove(dest)
        else:
            sys.exit('Cancelling operation...')
    shutil.move(src, dest)

class Config(object):
    def __init__(self):
        # Windows doesn't support binaries without .exe extension, sigh...
        rtmpdump_bin = 'rtmpdump'
        if sys.platform.startswith('win32'):
            rtmpdump_bin = 'rtmpdump.exe'

        self.config = SafeConfigParser(
            defaults={'video_quality': 'highest',
                      'language': 'English',
                      'result_path': '.',
                      'retry': '3',
                      'rtmpdump_path': rtmpdump_bin,
                      'proxy': 'None'})

    def load_config(self, config_path):
        if not os.path.exists(config_path):
            os.mkdir(config_path)
        self.config.read(os.path.join(config_path, 'settings.ini'))

        video_quality_dict = {'android': {'video_format': '107', 'resolution': '71'},
                        '360p': {'video_format': '106', 'resolution': '60'},
                        '480p': {'video_format': '106', 'resolution': '61'},
                        '720p': {'video_format': '106', 'resolution': '62'},
                        '1080p': {'video_format': '108', 'resolution': '80'},
                        'highest': {'video_format': '0', 'resolution': '0'}}
        video_quality = self.config.get('DEFAULT', 'video_quality')
        try:
            selected_quality = video_quality_dict[video_quality]
            self.video_format = selected_quality['video_format']
            self.resolution = selected_quality['resolution']
        except KeyError:
            sys.exit("Invalid 'video_quality' option in 'settings.ini'!")

        lang_dict = {'Espanol': 'Espanol',
                     'Espanol_Espana': 'Espanol (Espana)',
                     'Francais': 'Francais (France)',
                     'Portugues': 'Portugues (Brasil)',
                     'English': 'English|English (US)',
                     'Deutsch': 'Deutsch',
                     'Turkce': 'Turkce',
                     'Italiano': 'Italiano'}
        lang = self.config.get('DEFAULT', 'language')
        try:
            self.lang = lang_dict[lang]
        except KeyError:
            sys.exit("Invalid 'language' option in 'settings.ini'!")

        self.result_path = os.path.expanduser(self.config.get('DEFAULT', 'result_path'))
        if not os.path.isdir(self.result_path):
            sys.exit("Path {} don't exist or isn't a directory!".format(self.result_path))

        try:    
            self.retry = int(self.config.get('DEFAULT', 'retry'))
        except ValueError:
            sys.exit("Invalid 'retry' option in 'settings.ini!'")

        self.rtmpdump_path = os.path.expanduser(self.config.get('DEFAULT', 'rtmpdump_path'))
        try:
            # rtmpdump doesn't have a --version, sigh... At least -h return 0.
            subprocess.check_output([self.rtmpdump_path, '-h'], stderr=subprocess.STDOUT)
        except OSError:
            sys.exit("Could not start 'rtmpdump' from path '{}'. Check if 'rtmpdump' is installed and is in "
                     "your PATH or check 'rtmpdump_path' option in 'settings.ini'!".format(self.rtmpdump_path))
        
        self.cookies_path = os.path.join(config_path, 'cookies.txt')

        self.proxy = None
        self.proxy_type = None
        proxy = self.config.get('DEFAULT', 'proxy')
        if proxy is not 'None':
            self.proxy = urllib.parse.urlparse(proxy).netloc
            self.proxy_type = urllib.parse.urlparse(proxy).scheme

    def return_config(self):
        config = {'video_format': self.video_format, 'resolution': self.resolution, 'lang': self.lang,
                  'result_path': self.result_path, 'retry': self.retry, 'rtmpdump_path': self.rtmpdump_path,
                  'cookies_path': self.cookies_path, 'proxy': self.proxy, 'proxy_type': self.proxy_type,
                  'proxy': self.proxy}
        return config
